/*
 * Copyright 2012 Martin Skurla
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package org.crazyjavahacking.astvisualizer.impl.javac.util;


import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Arrays;
import java.util.List;

import javax.tools.JavaCompiler;
import javax.tools.JavaCompiler.CompilationTask;
import javax.tools.JavaFileObject;
import javax.tools.ToolProvider;

import com.sun.source.tree.CompilationUnitTree;
import com.sun.source.util.JavacTask;


/**
 * Java compiler utility class.
 *
 * @author Martin
 */
public final class JavaCompilerUtils {

    //-----------------------------------------------------------------------------------------------------------------
    // Constructors.
    //-----------------------------------------------------------------------------------------------------------------
    private JavaCompilerUtils() {}


    //-----------------------------------------------------------------------------------------------------------------
    // Public Methods.
    //-----------------------------------------------------------------------------------------------------------------
    /**
     * Returns CompilationTask from given JavaFileObject.
     *
     * @param javaFileObject JavaFileObject
     *
     * @return CompilationTask
     */
    public static JavaCompiler.CompilationTask getCompilationTaskFromJavaFileObject(JavaFileObject javaFileObject) {
        List<JavaFileObject> javaSources = Arrays.asList(javaFileObject);

        JavaCompiler javaCompiler = ToolProvider.getSystemJavaCompiler();

        JavaCompiler.CompilationTask compilationTask = javaCompiler.getTask(
                new OutputStreamWriter(System.out),
                null,  // default file manager
                null,  // default diagnostic listener
                null,  // no compiler options
                null,  // no classes for annotation processing
                javaSources);

        return compilationTask;
    }

    /**
     * Compiles given CompilationTaks.
     *
     * @param compilationTask CompilationTask
     *
     * @throws JavacParsingException if problem during javac parsing
     *
     * @return CompilationUnitTree
     */
    public static CompilationUnitTree compile(JavaCompiler.CompilationTask compilationTask) {
        JavacTask javacTask = (JavacTask) compilationTask;

        try {
            return javacTask.parse().iterator().next();
        }
        catch (IOException e) {
            throw new JavacParsingException(e);
        }
    }

    /**
     * Compiles given JavaFileObject.
     *
     * @param javaFileObject JavaFileObject
     *
     * @throws JavacParsingException if problem during javac parsing
     *
     * @return CompilationUnitTree
     */
    public static CompilationUnitTree compile(JavaFileObject javaFileObject) {
        CompilationTask compilationTask = getCompilationTaskFromJavaFileObject(javaFileObject);

        return compile(compilationTask);
    }


   //-----------------------------------------------------------------------------------------------------------------
    // Inner classes.
    //-----------------------------------------------------------------------------------------------------------------
    /**
     * Represents exceptional state during javac parsing.
     */
    public static final class JavacParsingException extends RuntimeException {
        private static final long serialVersionUID = 1L;

        public JavacParsingException(Throwable cause) {
            super(cause);
        }
    }
}
